from osgeo import gdal
from multiprocessing import Process, Pipe
import struct, os
import numpy as np

gdal.UseExceptions()
fmttypes = {'Byte':'B', 'UInt16':'H', 'Int16':'h', 'UInt32':'I', 'Int32':'i', 'Float32':'f', 'Float64':'d'}

"""
dataLoader

Class that reads data from a given input file and passes it to a Pipe object
designed to run as a separate process and inherits from Process module

ERROR: GEOTiff files do NOT work, they are corrupting at some point 
"""
class dataLoader(Process):

    """
    __init__

    paramaters:
        inputFile - must be a valid file path as a string
        output_pipe - a Pipe object to pass read information into
   
    opens the input file and grabs the header information
    sets several instance variables
    """
    def __init__(self, inputFile):
        Process.__init__(self)
        #self.output_pipe = output_pipe
        self.file_name = inputFile
        self._openFile()
        self._readHeaderInfo()
        self.cur_line=""
        self.prev_last_row=""

    """
    getHeaderInfo
    
    Returns header information as a six-tuple in this order:
    (ncols, nrows, cellsize, NODATA, xllcorner, yllcorner)
    """
    def getHeaderInfo(self):
        return self.totalCols, self.totalRows, self.cellsize, self.NODATA, self.xllcorner, self.yllcorner

    """
    _readHeaderInfo

    requires file to be opened already, gets header info from file and puts it in instance variables
    """
    def _readHeaderInfo(self):
            src_ds = gdal.Open(self.file_name) #open again to get GeoT info
            GeoT = src_ds.GetGeoTransform()
            
            # switch between these and it will fail when using open_file
            srcband = self.open_file
            #srcband = src_ds.GetRasterBand(1)
            # SEGMENTATION FAULT HERE
            print "In _readHeaderInfo"
            self.NODATA = srcband.GetNoDataValue()
            self.xllcorner = GeoT[0]
            self.yllcorner = GeoT[3]
            self.cellsize = srcband.GetScale()
            self.totalRows = srcband.YSize
            self.totalCols = srcband.XSize
            src_ds = None# close file
            print "Leaving _readHeaderInfo"
  
    """
    _openFile

    opens file_name and sets it to open_file, supports '.tif' and '.asc' files
    """
    def _openFile(self):
        f = gdal.Open(self.file_name)
        self.open_file = f.GetRasterBand(1)

    """
    run

    calls functions needed to read all data from file_name
    """
    def run(self):
        self._loadFunc() and going through 

    """
    _getLine

    returns a single row from the open file as a numpy float64 array
    """
    def _getLine(self, row):
        print "In _getLine"
                # ERROR
                # GDAL Raster File gets corrupted between opening and here
                # DOES NOT WORK
        val1 = self.open_file.DataType
        val2 = gdal.GetDataTypeName(self.open_file.DataType)
        print val1, val2
        # gets no farther than here
        f = struct.unpack(fmttypes[gdal.GetDataTypeName(self.open_file.DataType)]*self.totalCols, self.open_file.ReadRaster(0,row,self.open_file.XSize,1, buf_type=self.open_file.DataType))
        print f
        print "Leaving _getLine"      
        return np.float64(f)

    """
    _loadFunc

    sends data one row at a time to output_pipe, sends exactly the number of rows as are in the input file
    """
    def _loadFunc(self):
        count = 0
        while count < self.totalRows:
            #self.output_pipe.send(self._getLine(count))
            self._getLine(count)            
            count += 1
        #self.output_pipe.close()
        print "Input file loaded from disk"

if __name__ == '__main__':
    i=dataLoader("aigrid.tif")
    i.start()
    i.join()

